@@ -3,7 +3,7 @@
 """
 Job control on ARC0 resources.
 """
-# Copyright (C) 2009-2013 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,7 @@ Job control on ARC0 resources.
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.1.5 version (SVN $Revision: 3967 $)'
+__version__ = '2.2.0 version (SVN $Revision: 3972 $)'
 
 
 import sys
@@ -121,6 +121,13 @@ class ArcLrms(LRMS):
 
         self.targets_blacklist = []
 
+        log.warning(
+            "The ARC0 backend (used in resource '%s') is deprecated"
+            " and will be removed in the next major release of GC3Pie."
+            " Consider changing your configuration.",
+            name)
+
+
     @same_docstring_as(LRMS.cancel_job)
     @LRMS.authenticated
     def cancel_job(self, app):
@@ -484,7 +491,7 @@ class ArcLrms(LRMS):
 
     @same_docstring_as(LRMS.get_results)
     @LRMS.authenticated
-    def get_results(self, app, download_dir, overwrite=False):
+    def get_results(self, app, download_dir, overwrite=False, changed_only=True):
         jobid = app.execution.lrms_jobid
 
         # XXX: can raise encoding/decoding error if `download_dir`
@@ -496,53 +503,33 @@ class ArcLrms(LRMS):
         # directory, make a temporary directory for downloading files;
         # then move files to their final destination and delete the
         # temporary location.
-        tmp_download_dir = tempfile.mkdtemp(suffix='.d', dir=download_dir)
+        with (suffix='.d', dir=download_dir) as tmp_download_dir:
 
-        log.debug("Downloading %s output into temporary location '%s' ...", app, tmp_download_dir)
-        try:
-            jftpc = arclib.JobFTPControl()
-            jftpc.DownloadDirectory(jobid, tmp_download_dir)
-        except arclib.FTPControlError, ex:
-            # remove temporary download location
-            shutil.rmtree(tmp_download_dir, ignore_errors=True)
-            # FIXME: parsing error messages breaks if locale is not an
-            # English-based one!
-            if "Failed to allocate port for data transfer" in str(ex):
-                raise gc3libs.exceptions.RecoverableDataStagingError(
-                    "Recoverable Error: Failed downloading remote folder '%s': %s"
+           log.debug("Downloading %s output into temporary location '%s' ...",
+                      app, tmp_download_dir)
+            try:
+                jftpc = arclib.JobFTPControl()
+                jftpc.DownloadDirectory(jobid, tmp_download_dir)
+            except arclib.FTPControlError, ex:
+                # FIXME: parsing error messages breaks if locale is not an
+                # English-based one!
+                if "Failed to allocate port for data transfer" in str(ex):
+                    raise gc3libs.exceptions.RecoverableDataStagingError(
+                        "Recoverable Error:"
+                        " Failed downloading remote folder '%s': %s"
+                        % (jobid, str(ex)))
+                # critical error. consider job remote data as lost
+                raise gc3libs.exceptions.UnrecoverableDataStagingError(
+                    "Unrecoverable Error:"
+                    " Failed downloading remote folder '%s': %s"
                     % (jobid, str(ex)))
-            # critical error. consider job remote data as lost
-            raise gc3libs.exceptions.UnrecoverableDataStagingError(
-                "Unrecoverable Error: Failed downloading remote folder '%s': %s"
-                % (jobid, str(ex)))
 
-        log.debug("Moving %s output into download location '%s' ...", app, download_dir)
-        entries = os.listdir(tmp_download_dir)
-        if not overwrite:
-            # raise an early error before we start mixing files from
-            # the old and new download directories
-            for entry in entries:
-                dst = os.path.join(download_dir, entry)
-                if os.path.exists(dst):
-                    # remove temporary download location
-                    shutil.rmtree(tmp_download_dir, ignore_errors=True)
-                    gc3libs.log.warning(
-                        "Entry '%s' in download directory '%s' already exists,"
-                        " and no overwriting was requested."
-                        % (entry, download_dir))
-        # move all entries to the final destination
-        for entry in entries:
-            src = os.path.join(tmp_download_dir, entry)
-            dst = os.path.join(download_dir, entry)
-            if os.path.isdir(dst):
-                shutil.rmtree(dst)
-            os.rename(src, dst)
+            log.debug("Moving %s output into download location '%s' ...",
+                      app, download_dir)
+            movetree(tmp_download_dir, download_dir, overwrite, changed_only)
 
-        # remove temporary download location (XXX: is it correct to ignore errors here?)
-        shutil.rmtree(tmp_download_dir, ignore_errors=True)
-
-        app.execution.download_dir = download_dir
-        return
+            # all done, update application
+            app.execution.download_dir = download_dir
 
 
     @same_docstring_as(LRMS.free)
