@@ -4,7 +4,7 @@
 Job control using ``libarcclient``.  (Which can submit to all
 EMI-supported resources.)
 """
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -21,7 +21,7 @@ EMI-supported resources.)
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.1.5 version (SVN $Revision$)'
+__version__ = '2.2.0 version (SVN $Revision$)'
 
 
 import sys
@@ -65,7 +65,7 @@ class Arc1Lrms(LRMS):
 
         log.warning(
             "The ARC1 backend (used in resource '%s') is deprecated"
-            " and will be removed in a future release."
+            " and will be removed in the next major release of GC3Pie."
             " Consider changing your configuration.",
             name)
 
@@ -482,7 +482,7 @@ class Arc1Lrms(LRMS):
 
     @same_docstring_as(LRMS.get_results)
     @LRMS.authenticated
-    def get_results(self, app, download_dir, overwrite=False):
+    def get_results(self, app, download_dir, overwrite=False, changed_only=True):
         jobid = app.execution.lrms_jobid
 
         # XXX: can raise encoding/decoding error if `download_dir`
@@ -496,62 +496,40 @@ class Arc1Lrms(LRMS):
         # directory, make a temporary directory for downloading files;
         # then move files to their final destination and delete the
         # temporary location.
-        tmp_download_dir = tempfile.mkdtemp(suffix='.d', dir=download_dir)
+        with tempdir(suffix='.d', dir=download_dir) as tmp_download_dir:
 
-        log.debug("Downloading %s output into temporary location '%s' ...", app, tmp_download_dir)
+            log.debug("Downloading %s output into temporary location '%s' ...",
+                      app, tmp_download_dir)
 
-        # Get a list of downloadable files
-        download_file_list = c.GetDownloadFiles(j.JobID);
+            # Get a list of downloadable files
+            download_file_list = c.GetDownloadFiles(j.JobID);
 
-        source_url = arc.URL(j.JobID.str())
-        destination_url = arc.URL(tmp_download_dir)
+            source_url = arc.URL(j.JobID.str())
+            destination_url = arc.URL(tmp_download_dir)
 
-        source_path_prefix = source_url.Path()
-        destination_path_prefix = destination_url.Path()
+            source_path_prefix = source_url.Path()
+            destination_path_prefix = destination_url.Path()
 
-        errors = 0
-        for remote_file in download_file_list:
-            source_url.ChangePath(os.path.join(source_path_prefix,remote_file))
-            destination_url.ChangePath(os.path.join(destination_path_prefix,remote_file))
-            if not c.ARCCopyFile(source_url,destination_url):
-                log.warning("Failed downloading '%s' to '%s'",
-                            source_url.str(), destination_url.str())
-                errors += 1
-        if errors > 0:
-            # remove temporary download location
-            shutil.rmtree(tmp_download_dir, ignore_errors=True)
-            raise gc3libs.exceptions.UnrecoverableDataStagingError(
-                "Failed downloading remote folder of job '%s' into '%s'."
-                " There were %d errors, reported at the WARNING level in log files."
-                % (jobid, download_dir, errors))
+            errors = 0
+            for remote_file in download_file_list:
+                source_url.ChangePath(os.path.join(source_path_prefix,remote_file))
+                destination_url.ChangePath(os.path.join(destination_path_prefix,remote_file))
+                if not c.ARCCopyFile(source_url,destination_url):
+                    log.warning("Failed downloading '%s' to '%s'",
+                                source_url.str(), destination_url.str())
+                    errors += 1
+            if errors > 0:
+                raise gc3libs.exceptions.UnrecoverableDataStagingError(
+                    "Failed downloading remote folder of job '%s' into '%s'."
+                    " There were %d errors, reported at the WARNING level in log files."
+                    % (jobid, download_dir, errors))
 
-        log.debug("Moving %s output into download location '%s' ...", app, download_dir)
-        entries = os.listdir(tmp_download_dir)
-        if not overwrite:
-            # raise an early error before we start mixing files from
-            # the old and new download directories
-            for entry in entries:
-                dst = os.path.join(download_dir, entry)
-                if os.path.exists(entry):
-                    # remove temporary download location
-                    shutil.rmtree(tmp_download_dir, ignore_errors=True)
-                    raise gc3libs.exceptions.UnrecoverableDataStagingError(
-                        "Entry '%s' in download directory '%s' already exists,"
-                        " and no overwriting was requested."
-                        % (entry, download_dir))
-        # move all entries to the final destination
-        for entry in entries:
-            src = os.path.join(tmp_download_dir, entry)
-            dst = os.path.join(download_dir, entry)
-            if os.path.isdir(dst):
-                shutil.rmtree(dst)
-            os.rename(src, dst)
+            log.debug("Moving %s output into download location '%s' ...",
+                      app, download_dir)
+            movetree(tmp_download_dir, download_dir, overwrite, changed_only)
 
-        # remove temporary download location (XXX: is it correct to ignore errors here?)
-        shutil.rmtree(tmp_download_dir, ignore_errors=True)
-
-        app.execution.download_dir = download_dir
-        return
+            # all done, update application
+            app.execution.download_dir = download_dir
 
 
     @same_docstring_as(LRMS.free)
