@@ -4,7 +4,7 @@
 Job control on PBS/Torque clusters (possibly connecting to the
 front-end via SSH).
 """
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -21,7 +21,7 @@ front-end via SSH).
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.1.5 version (SVN $Revision$)'
+__version__ = '2.2.0 version (SVN $Revision$)'
 
 
 import datetime
@@ -52,7 +52,7 @@ import batch
 
 # regexps for extracting relevant strings
 
-_qsub_jobid_re = re.compile(r'(?P<jobid>\d+)', re.I)
+_qsub_jobid_re = re.compile(r'(?P<jobid>\d+).*', re.I)
 
 _qstat_line_re = re.compile(
     r'^(?P<jobid>\d+)[^\d]+\s+'
@@ -176,7 +176,6 @@ def count_jobs(qstat_output, whoami):
     own_queued = 0
     n = 0
     for line in qstat_output.split('\n'):
-        # import pdb; pdb.set_trace()
         log.info("Output line: %s" % line)
         m = _qstat_line_re.match(line)
         if not m:
@@ -209,9 +208,12 @@ class PbsLrms(batch.BatchSystem):
                  auth,  # ignored if `transport` is 'local'
                  # these are inherited from `BatchSystem`
                  frontend, transport,
-                 accounting_delay=15,
                  # these are specific to this backend
                  queue=None,
+                 # (Note that optional arguments to the `BatchSystem` class, e.g.:
+                 #     keyfile=None, accounting_delay=15,
+                 # are collected into `extra_args` and should not be explicitly
+                 # spelled out in this signature.)
                  **extra_args):
 
         # init base class
@@ -219,7 +221,7 @@ class PbsLrms(batch.BatchSystem):
             self, name,
             architecture, max_cores, max_cores_per_job,
             max_memory_per_core, max_walltime, auth,
-            frontend, transport, accounting_delay=accounting_delay,
+            frontend, transport,
             **extra_args)
 
         # backend-specific setup
@@ -264,7 +266,7 @@ class PbsLrms(batch.BatchSystem):
             jobstatus['state'] = Run.State.RUNNING
         elif job_status in ['S', 'H', 'T'] or 'qh' in job_status:
             jobstatus['state'] = Run.State.STOPPED
-        elif job_status in ['C', 'E']:
+        elif job_status in ['C', 'E', 'F']:
             jobstatus['state'] = Run.State.TERMINATING
         else:
             jobstatus['state'] = Run.State.UNKNOWN
