@@ -19,7 +19,7 @@ in core Python, namely:
   this indicates a bug in the software.
 
 """
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -36,7 +36,7 @@ in core Python, namely:
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.1.5 version (SVN $Revision: 3967 $)'
+__version__ = '2.2.0 version (SVN $Revision: 3972 $)'
 
 
 import gc3libs
@@ -115,6 +115,7 @@ class AuthError(Error):
     """
     pass
 
+
 class RecoverableAuthError(AuthError, RecoverableError):
     pass
 
@@ -176,6 +177,15 @@ class InternalError(Error):
     """
     pass
 
+class AuxiliaryCommandError(InternalError):
+    """
+    Raised when some external command that we depend upon has failed.
+
+    For instance, we might need to list processes on a remote machine
+    but ``ps aux`` does not run because of insufficient privileges.
+    """
+    pass
+
 
 class InvalidArgument(Error, AssertionError): # XXX: should this be fatal?
     """
@@ -247,16 +257,26 @@ class LRMSError(Error):
     pass
 
 
-class LRMSSubmitError(Error):
+class LRMSSubmitError(LRMSError):
     pass
 
-class LRMSSkipSubmissionToNextIteration(RecoverableError):
+
+class LRMSSkipSubmissionToNextIteration(LRMSSubmitError, RecoverableError):
     """
-    Tells the Engine to skip submission of a job until the next
-    iteration.
+    An elastic resource has initiated adapting for a new task.
+    Although we cannot submit the task right now, it *will* be
+    accepted in the (not too distant) future.
     """
     pass
 
+
+class MaximumCapacityReached(LRMSSubmitError, RecoverableError):
+    """
+    Indicates that a resource is full and cannot run any more jobs.
+    """
+    pass
+
+
 class NoConfigurationFile(FatalError):
     """
     Raised when the configuration file cannot be read (e.g., does not
