@@ -152,6 +152,7 @@ class Session(list):
 
         """
         self.path = os.path.abspath(path)
+        self.name = os.path.basename(self.path)
         self.tasks = dict()
         # Session not yet created
         self.created = -1
@@ -172,7 +173,7 @@ class Session(list):
         if os.path.isdir(self.path):
             # Session already exists?
             try:
-                self._load_session()
+                self._load_session(**extra_args)
             except IOError, err:
                 gc3libs.log.debug("Cannot load session '%s': %s", path, str(err))
                 if err.errno == 2:  # "No such file or directory"
@@ -329,9 +330,9 @@ class Session(list):
         """
         try:
             store_fname = os.path.join(self.path, self.STORE_URL_FILENAME)
-            self.store_url = gc3libs.utils.read_contents(store_fname)
+            self.store_url = gc3libs.utils.read_contents(store_fname).strip()
         except IOError:
-            gc3libs.log.error(
+            gc3libs.log.info(
                 "Unable to load session: file %s is missing." % (store_fname))
             raise
         self.store = gc3libs.persistence.make_store(self.store_url, **extra_args)
