@@ -10,7 +10,7 @@ patterns of job group execution; they can be combined to form more
 complex workflows.  Hook methods are provided so that derived classes
 can implement problem-specific job control policies.
 """
-# Copyright (C) 2009-2013 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -27,7 +27,7 @@ can implement problem-specific job control policies.
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.1.5 version (SVN $Revision$)'
+__version__ = '2.2.0 version (SVN $Revision$)'
 
 import time
 import os
@@ -109,7 +109,7 @@ class TaskCollection(Task):
     # task execution manipulation -- these methods should be overriden
     # in derived classes, to implement the desired policy.
 
-    def submit(self, resubmit=False, **extra_args):
+    def submit(self, resubmit=False, targets=None, **extra_args):
         raise NotImplementedError("Called abstract method TaskCollection.submit() - this should be overridden in derived classes.")
 
 
@@ -126,7 +126,8 @@ class TaskCollection(Task):
         raise NotImplementedError("Called abstract method TaskCollection.kill() - this should be overridden in derived classes.")
 
 
-    def fetch_output(self, output_dir=None, overwrite=False, **extra_args):
+    def fetch_output(self, output_dir=None,
+                     overwrite=False, changed_only=True, **extra_args):
         # if `output_dir` is not None, it is interpreted as the base
         # directory where to download files; each task will get its
         # own subdir based on its `.persistent_id`
@@ -146,6 +147,7 @@ class TaskCollection(Task):
                 task,
                 task_output_dir,
                 overwrite,
+                changed_only,
                 **extra_args)
         # if any sub-task is not yet TERMINATED, return the base
         # output directory for the collection...
@@ -309,7 +311,7 @@ class SequentialTaskCollection(TaskCollection):
             return Run.State.RUNNING
 
 
-    def submit(self, resubmit=False, **extra_args):
+    def submit(self, resubmit=False, targets=None, **extra_args):
         """
         Start the current task in the collection.
         """
@@ -317,7 +319,7 @@ class SequentialTaskCollection(TaskCollection):
             self._current_task = 0
         task = self.tasks[self._current_task]
         task.attach(self._controller)
-        task.submit(resubmit, **extra_args)
+        task.submit(resubmit, targets, **extra_args)
         if task.execution.state == Run.State.NEW:
             # submission failed, state unchanged
             self.execution.state = Run.State.NEW
@@ -521,12 +523,12 @@ class ParallelTaskCollection(TaskCollection):
         return [ task.progress() for task in self.tasks ]
 
 
-    def submit(self, resubmit=False, **extra_args):
+    def submit(self, resubmit=False, targets=None, **extra_args):
         """
         Start all tasks in the collection.
         """
         for task in self.tasks:
-            task.submit(resubmit, **extra_args)
+            task.submit(resubmit, targets, **extra_args)
         self.execution.state = self._state()
 
 
@@ -586,6 +588,7 @@ class ChunkedParameterSweep(ParallelTaskCollection):
         but also creates new tasks if less than
         `chunk_size` are running.
         """
+        ParallelTaskCollection.update_state(self, **extra_args)
         # XXX: proposal, reset chuck_size from self._controller.max_in_flight
         # this is the way to pass new 'max-running' value to the class
         # this creates though, a tigh coupling with 'controller' and maybe
@@ -611,9 +614,9 @@ class ChunkedParameterSweep(ParallelTaskCollection):
             for param in range(self._floor, top, self.step):
                 self.add(self.new_task(param, **extra_args))
             self._floor = top
+            self.execution.state = self._state()
             self.changed = True
-        return ParallelTaskCollection.update_state(self, **extra_args)
-
+        return self.execution.state
 
 class RetryableTask(Task):
     """
@@ -707,12 +710,16 @@ class RetryableTask(Task):
 
     def kill(self, **extra_args):
         self.task.kill(**extra_args)
+        if self.task.execution.state == Run.State.TERMINATED:
+            self.execution.state = Run.State.TERMINATED
+            self.changed = True
 
     def peek(self, *args, **extra_args):
         return self.task.peek(*args, **extra_args)
 
-    def submit(self, resubmit=False, **extra_args):
-        self.task.submit(**extra_args)
+    def submit(self, resubmit=False, targets=None, **extra_args):
+        # XXX: previous code ignored the `resubmit` argument; why?!
+        self.task.submit(resubmit, targets, **extra_args)
         # immediately update state if submission of managed task was successful;
         # otherwise this task may remain in ``NEW`` state which causes an
         # unwanted resubmission if the managing programs ends or is interrupted
